Added missing calls to safe_fname() and full_fname().
[rsync/rsync.git] / flist.c
diff --git a/flist.c b/flist.c
index eea7896..de27934 100644 (file)
--- a/flist.c
+++ b/flist.c
@@ -153,14 +153,14 @@ static void list_file_entry(struct file_struct *f)
                rprintf(FINFO, "%s %11.0f %s %s -> %s\n",
                        perms,
                        (double)f->length, timestring(f->modtime),
-                       f_name(f), f->u.link);
+                       safe_fname(f_name(f)), safe_fname(f->u.link));
        } else
 #endif
        {
                rprintf(FINFO, "%s %11.0f %s %s\n",
                        perms,
                        (double)f->length, timestring(f->modtime),
-                       f_name(f));
+                       safe_fname(f_name(f)));
        }
 }
 
@@ -194,7 +194,7 @@ static int readlink_stat(const char *path, STRUCT_STAT *buffer, char *linkbuf)
                if (copy_unsafe_links && unsafe_symlink(linkbuf, path)) {
                        if (verbose > 1) {
                                rprintf(FINFO,"copying unsafe symlink \"%s\" -> \"%s\"\n",
-                                       path, linkbuf);
+                                       safe_fname(path), safe_fname(linkbuf));
                        }
                        return do_stat(path, buffer);
                }
@@ -411,11 +411,11 @@ void send_file_entry(struct file_struct *file, int f, unsigned short base_flags)
 
        /* We must make sure we don't send a zero flag byte or the
         * other end will terminate the flist transfer.  Note that
* the use of XMIT_DEL_START on a non-dir has no meaning, so
         * it's harmless way to add a bit to the first flag byte. */
        if (protocol_version >= 28) {
                if (!flags && !S_ISDIR(mode))
-                       flags |= XMIT_DEL_START;
+                       flags |= XMIT_TOP_DIR;
                if ((flags & 0xFF00) || !flags) {
                        flags |= XMIT_EXTENDED_FLAGS;
                        write_byte(f, flags);
@@ -424,7 +424,7 @@ void send_file_entry(struct file_struct *file, int f, unsigned short base_flags)
                        write_byte(f, flags);
        } else {
                if (!(flags & 0xFF) && !S_ISDIR(mode))
-                       flags |= XMIT_DEL_START;
+                       flags |= XMIT_TOP_DIR;
                if (!(flags & 0xFF))
                        flags |= XMIT_LONG_NAME;
                write_byte(f, flags);
@@ -554,7 +554,7 @@ static void receive_file_entry(struct file_list *flist, int ndx,
        if (l2 >= MAXPATHLEN - l1) {
                rprintf(FERROR,
                        "overflow: flags=0x%x l1=%d l2=%d lastname=%s\n",
-                       flags, l1, l2, lastname);
+                       flags, l1, l2, safe_fname(lastname));
                overflow("receive_file_entry");
        }
 
@@ -659,17 +659,17 @@ static void receive_file_entry(struct file_list *flist, int ndx,
                file->dir.depth = 1;
 
        if (S_ISDIR(mode)) {
-               if (basename_len == 1+1 && *basename == '.') /* N.B. null */
+               if (basename_len == 1+1 && *basename == '.') /* +1 for '\0' */
                        file->dir.depth--;
-               if (flags & XMIT_DEL_START) {
+               if (flags & XMIT_TOP_DIR) {
                        in_del_hier = 1;
                        del_hier_name_len = file->dir.depth == 0 ? 0 : l1 + l2;
-                       file->flags |= FLAG_DEL_START;
-               } else if (delete_during && in_del_hier) {
+                       file->flags |= FLAG_TOP_DIR | FLAG_DEL_HERE;
+               } else if (in_del_hier) {
                        if (!relative_paths || !del_hier_name_len
                         || (l1 >= del_hier_name_len
                          && thisname[del_hier_name_len] == '/'))
-                               file->flags |= FLAG_DEL_START;
+                               file->flags |= FLAG_DEL_HERE;
                        else
                                in_del_hier = 0;
                }
@@ -772,7 +772,8 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
 
        if (strlcpy(thisname, fname, sizeof thisname)
            >= sizeof thisname - flist_dir_len) {
-               rprintf(FINFO, "skipping overly long name: %s\n", fname);
+               rprintf(FINFO, "skipping overly long name: %s\n",
+                       safe_fname(fname));
                return NULL;
        }
        clean_fname(thisname, 0);
@@ -817,7 +818,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
                goto skip_filters;
 
        if (S_ISDIR(st.st_mode) && !xfer_dirs) {
-               rprintf(FINFO, "skipping directory %s\n", thisname);
+               rprintf(FINFO, "skipping directory %s\n", safe_fname(thisname));
                return NULL;
        }
 
@@ -843,7 +844,7 @@ skip_filters:
 
        if (verbose > 2) {
                rprintf(FINFO, "[%s] make_file(%s,*,%d)\n",
-                       who_am_i(), thisname, filter_level);
+                       who_am_i(), safe_fname(thisname), filter_level);
        }
 
        if ((basename = strrchr(thisname, '/')) != NULL) {
@@ -1050,7 +1051,7 @@ static void send_directory(int f, struct file_list *flist,
        if (errno) {
                io_error |= IOERR_GENERAL;
                *p = '\0';
-               rsyserr(FERROR, errno, "readdir(%s)", fbuf);
+               rsyserr(FERROR, errno, "readdir(%s)", full_fname(fbuf));
        }
 
        closedir(d);
@@ -1139,7 +1140,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                }
 
                if (S_ISDIR(st.st_mode) && !xfer_dirs) {
-                       rprintf(FINFO, "skipping directory %s\n", fname);
+                       rprintf(FINFO, "skipping directory %s\n",
+                               safe_fname(fname));
                        continue;
                }
 
@@ -1216,7 +1218,7 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                        filesystem_dev = st.st_dev;
 
                do_subdirs = recurse >= 1 ? recurse-- : recurse;
-               send_file_name(f, flist, fname, do_subdirs, XMIT_DEL_START);
+               send_file_name(f, flist, fname, do_subdirs, XMIT_TOP_DIR);
 
                if (olddir[0]) {
                        flist_dir = NULL;
@@ -1318,7 +1320,7 @@ struct file_list *recv_file_list(int f)
 
                if (verbose > 2) {
                        rprintf(FINFO, "recv_file_name(%s)\n",
-                               f_name(flist->files[i]));
+                               safe_fname(f_name(flist->files[i])));
                }
        }
        receive_file_entry(NULL, 0, 0, 0); /* Signal that we're done. */
@@ -1474,13 +1476,12 @@ static void clean_flist(struct file_list *flist, int strip_root, int no_dups)
                        if (verbose > 1 && !am_server) {
                                rprintf(FINFO,
                                        "removing duplicate name %s from file list %d\n",
-                                       f_name(flist->files[i]), i);
+                                       safe_fname(f_name(flist->files[i])), i);
                        }
                        /* Make sure that if we unduplicate '.', that we don't
-                        * lose track of a user-specified starting point (or
-                        * else deletions will mysteriously fail with -R). */
-                       if (flist->files[i]->flags & FLAG_DEL_START)
-                               flist->files[prev_i]->flags |= FLAG_DEL_START;
+                        * lose track of a user-specified top directory. */
+                       if (flist->files[i]->flags & FLAG_TOP_DIR)
+                               flist->files[prev_i]->flags |= FLAG_TOP_DIR;
 
                        clear_file(i, flist);
                } else
@@ -1528,7 +1529,7 @@ static void output_flist(struct file_list *flist, const char *whose_list)
                        sprintf(depthbuf, "%d", file->dir.depth);
                rprintf(FINFO, "[%s] i=%d %s %s%s%s%s mode=0%o len=%.0f%s%s flags=%x\n",
                        whose_list, i, am_sender ? NS(file->dir.root) : depthbuf,
-                       file->dirname ? file->dirname : "",
+                       file->dirname ? safe_fname(file->dirname) : "",
                        file->dirname ? "/" : "", NS(file->basename),
                        S_ISDIR(file->mode) ? "/" : "", (int)file->mode,
                        (double)file->length, uidbuf, gidbuf, file->flags);
@@ -1655,13 +1656,19 @@ static int is_backup_file(char *fn)
 }
 
 
-/* This function is used to implement --delete-during. */
-void delete_in_dir(struct file_list *flist, char *fbuf, int dlen, int new_depth)
+/* This function is used to implement per-directory deletion, and
+ * is used by all the --delete-WHEN options.  Note that the fbuf
+ * pointer must point to a MAXPATHLEN buffer with the name of the
+ * directory in it (the functions we call will append names onto
+ * the end, but the old dir value will be restored on exit). */
+void delete_in_dir(struct file_list *flist, char *fbuf,
+                  struct file_struct *file)
 {
        static int min_depth = MAXPATHLEN, cur_depth = -1;
        static void *filt_array[MAXPATHLEN/2+1];
        struct file_list *dir_list;
        STRUCT_STAT st;
+       int dlen;
 
        if (!flist) {
                while (cur_depth >= min_depth)
@@ -1670,7 +1677,7 @@ void delete_in_dir(struct file_list *flist, char *fbuf, int dlen, int new_depth)
                cur_depth = -1;
                return;
        }
-       if (new_depth >= MAXPATHLEN/2+1)
+       if (file->dir.depth >= MAXPATHLEN/2+1)
                return; /* Impossible... */
 
        if (max_delete && deletion_count >= max_delete)
@@ -1683,17 +1690,18 @@ void delete_in_dir(struct file_list *flist, char *fbuf, int dlen, int new_depth)
                return;
        }
 
-       while (cur_depth >= new_depth && cur_depth >= min_depth)
+       while (cur_depth >= file->dir.depth && cur_depth >= min_depth)
                pop_local_filters(filt_array[cur_depth--]);
-       cur_depth = new_depth;
+       cur_depth = file->dir.depth;
        if (min_depth > cur_depth)
                min_depth = cur_depth;
+       dlen = strlen(fbuf);
        filt_array[cur_depth] = push_local_filters(fbuf, dlen);
 
        if (link_stat(fbuf, &st, keep_dirlinks) < 0)
                return;
 
-       if (one_file_system)
+       if (one_file_system && file->flags & FLAG_TOP_DIR)
                filesystem_dev = st.st_dev;
 
        dir_list = flist_new(WITHOUT_HLINK, "delete_in_dir");
@@ -1740,13 +1748,10 @@ void delete_missing(struct file_list *full_list, struct file_list *dir_list,
                                        rprintf(FINFO, "deleting %s\n",
                                                safe_fname(f));
                                }
-                       } else if (S_ISDIR(mode)) {
-                               int dflag = delete_during ? DEL_FORCE_RECURSE
-                                                         : DEL_NO_RECURSE;
-                               delete_file(f, DEL_DIR | dflag);
-                       } else {
+                       } else if (S_ISDIR(mode))
+                               delete_file(f, DEL_DIR | DEL_FORCE_RECURSE);
+                       else
                                delete_file(f, 0);
-                       }
                        deletion_count++;
                        if (max_delete && deletion_count >= max_delete)
                                break;
@@ -411,11 +411,11 @@ void send_file_entry(struct file_struct *file, int f, unsigned short base_flags)
        /* We must make sure we don't send a zero flag byte or the
         * other end will terminate the flist transfer.  Note that
-        * the use of XMIT_DEL_START on a non-dir has no meaning, so
+        * the use of XMIT_TOP_DIR on a non-dir has no meaning, so
         * it's harmless way to add a bit to the first flag byte. */
        if (protocol_version >= 28) {